#include <sta/devices/arduino/bus/spi.hpp>

#include <sta/lang.hpp>

#include <sta/devices/arduino/hal.hpp>

namespace sta
{
    ArduinoSPI::ArduinoSPI(const SPISettings & settings, Mutex * mutex /* = nullptr */)
        : SPI{ settings, mutex }
    {
        SPIClass::begin();
    }

    void ArduinoSPI::transfer(uint8_t value)
    {
        SPIClass::transfer(value);
    }

    void ArduinoSPI::transfer16(uint16_t value)
    {
        SPIClass::transfer16(value);
    }

    void ArduinoSPI::transfer(const uint8_t * buffer, size_t size)
    {
        // We have to create a temporary buffer here because the buffer argument in SPIClass::transfer is not const.
        uint8_t temp_buffer[size];
        memcpy(temp_buffer, buffer, size);

        SPIClass::transfer(temp_buffer, size);
    }

    void ArduinoSPI::transfer(const uint8_t * txBuffer, uint8_t * rxBuffer, size_t size)
    {
        // Copy txBuffer content into rxBuffer, SPIClass::transfer will store received values in rxBuffer.
        memcpy(rxBuffer, txBuffer, size);

        SPIClass::transfer(rxBuffer, size);
    }

    void ArduinoSPI::receive(uint8_t * buffer, size_t size)
    {
        SPIClass::transfer(buffer, size);
    }

    void ArduinoSPI::fill(uint8_t value, size_t count) 
    {
        uint8_t temp_buffer[count];
        memset(temp_buffer, value, count);

        SPIClass::transfer(temp_buffer, count);
    }

    void ArduinoSPI::acquire()
    {
        SPI::acquire();

        const SPISettings & set = settings();

        uint8_t dataMode;
        switch (set.mode)
        {
        case SPIMode::MODE_0:
            dataMode = SPI_MODE0;
            break;
        case SPIMode::MODE_1:
            dataMode = SPI_MODE1;
            break;
        case SPIMode::MODE_2:
            dataMode = SPI_MODE2;
            break;
        case SPIMode::MODE_3:
            dataMode = SPI_MODE3;
            break;        
        default:
            STA_UNREACHABLE();
            break;
        }

        uint8_t bitOrder;
        switch (set.bitOrder)
        {
        case SPIBitOrder::MSB:
            bitOrder = MSBFIRST;
            break;
        case SPIBitOrder::LSB:
            bitOrder = LSBFIRST;
            break;
        default:
            STA_UNREACHABLE();
            break;
        }

        hal::SPISettings halSettings(set.clkSpeed, bitOrder, dataMode);

        SPIClass::beginTransaction(halSettings);
    }

    void ArduinoSPI::release()
    {
        SPIClass::endTransaction();

        SPI::release();
    }

    ArduinoSPIDevice::ArduinoSPIDevice(ArduinoSPI * intf, ArduinoGpioPin * csPin)
        : SPIDevice{intf, csPin}
    {
        csPin->setState(GpioPinState::GPIO_HIGH);
    }
} // namespace sta
